@@ -71,6 +71,16 @@ struct wt_status {
 	struct string_list ignored;
 };
 
+struct wt_status_state {
+	int merge_in_progress;
+	int am_in_progress;
+	int am_empty_patch;
+	int rebase_in_progress;
+	int rebase_interactive_in_progress;
+	int cherry_pick_in_progress;
+	int bisect_in_progress;
+};
+
 void wt_status_prepare(struct wt_status *s);
 void wt_status_print(struct wt_status *s);
 void wt_status_collect(struct wt_status *s);
@@ -82,7 +92,5 @@ void status_printf_ln(struct wt_status *s, const char *color, const char *fmt, .
 	;
 void status_printf(struct wt_status *s, const char *color, const char *fmt, ...)
 	;
-void status_printf_more(struct wt_status *s, const char *color, const char *fmt, ...)
-	__attribute__((format(printf, 3, 4)));
 
 #endif /* STATUS_H */
